﻿Imports System.Data.SqlClient   'Untuk proses ke database
Imports System.IO               'Untuk proses penyimpanan log error

Module PublicFunctions
    'Deklarasikan DBConnection sebagai object dari SQLConnection dengan string koneksi dari Database1ConnectionString
    Public DBConnection As New SqlConnection(My.Settings.Database1ConnectionString)
    Public ErrorLog As Boolean = True   'Aktifkan error log
    Public lastSQL As String = ""       'String perintah sql terakhir

    ''' <summary>
    ''' Percobaan untuk membuka koneksi ke dalam database
    ''' </summary>
    ''' <param name="RetMessage">Pesan error yang akan ditampilkan</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function OpenConnection(ByRef RetMessage As String) As SqlConnection
        Try
            DBConnection.Open()
        Catch ex As SqlException
            RetMessage = ex.Message
            Return Nothing
        Catch ex As Exception
            RetMessage = ex.Message
            Return Nothing
        End Try
        Return DBConnection

    End Function

    ''' <summary>
    ''' Menjalankan perintah SQL dan mengembalikan hasilnya sebagai dataview
    ''' </summary>
    ''' <param name="str">String perintah SQL</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function DoSqlDataView(ByVal str As String) As DataView

        Try
            Dim da As SqlDataAdapter

            Dim ds As DataSet
            Dim dv As DataView

            da = New SqlDataAdapter(str, DBConnection)
            ds = New DataSet
            da.Fill(ds)
            dv = New DataView(ds.Tables(0))
            Return dv
        Catch e As Exception
            'Jika terjadi kesalahan, simpan pesan kesalahan ke dalam file
            saveerrormessage(e.Message, e.StackTrace, e.Source)
        End Try
        Return New DataView

    End Function

    ''' <summary>
    ''' Menjalankan perintah SQL
    ''' </summary>
    ''' <param name="str">String perintah SQL yang akan dijalankan</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function DoSqlCommand(ByVal str As String) As Boolean
        Try
            lastSQL = str

            Dim cm As SqlCommand
            cm = New SqlCommand(str, DBConnection)
            cm.ExecuteNonQuery()
            Return True
        Catch e As Exception
            'Jika terjadi kesalahan, simpan pesan kesalahan ke dalam file 
            saveerrormessage(e.Message, e.StackTrace, e.Source)
            Return False
        End Try

    End Function

    ''' <summary>
    ''' Mendapatkan nilai IDENTITY dari field autoincrement pada suatu tabel
    ''' </summary>
    ''' <param name="pFromTableName">Nama tabel</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetIdentity(ByVal pFromTableName As String) As Integer
        Dim str As String
        Dim dv As DataView
        Dim iIdentity As Integer
        str = "SELECT @@IDENTITY as myValue FROM " & pFromTableName
        dv = DoSqlDataView(str)
        If dv.Count > 0 Then
            iIdentity = CInt(dv(0)(0))
        End If
        Return iIdentity
    End Function


    ''' <summary>
    ''' Mengganti tanda ' ke '' dan string kosong ke "". Digunakan saat menyiapkan perintah SQL
    ''' </summary>
    ''' <param name="strValue"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function PrepareStr(ByVal strValue As String) As String
        Try
            If strValue.Trim() = "" Then
                Return "''"
            Else
                strValue = strValue.Replace("'", "''")
                Return "'" & strValue.Trim & "'"
            End If
        Catch e As Exception
            saveerrormessage(e.Message, e.StackTrace, e.Source)
            Return "''"
        End Try

    End Function

    ''' <summary>
    ''' Untuk menyimpan pesan kesalahan selama program dijalankan
    ''' </summary>
    ''' <param name="Message">Pesan kesalahan</param>
    ''' <param name="stacktrace">Lokasi kesalahan</param>
    ''' <param name="source">Object penyebab kesalahan</param>
    ''' <remarks></remarks>
    Public Sub saveerrormessage(ByVal Message As String, ByVal stacktrace As String, ByVal source As String)
        If ErrorLog = True Then
            Dim errout As System.IO.StreamWriter
            Dim errfilename As String = "RtError.txt"   'RtError merupakan nama file yang akan digunakan
            If Not File.Exists(errfilename) Then
                'file untuk menyimpan log error belum ada, buat baru..
                errout = File.CreateText(errfilename)
            Else
                errout = File.AppendText(errfilename)
            End If

            'Buka dan tulis pesan error ke dalam file
            errout.WriteLine("******* TutorialVB.NET RUNTIME ERROR *******")
            errout.WriteLine("Date :" + CStr(Now()))
            errout.WriteLine("MSG  :" + Message)
            errout.WriteLine("SQL  :" + lastSQL)
            errout.WriteLine("SOURCE :" + source)
            errout.WriteLine("STACK  :" + stacktrace)
            errout.Flush()
            errout.Close()
        End If

    End Sub
End Module
